@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# $Id: viewlet.py 76036 2008-11-18 09:55:26Z glenfant $
+# $Id: viewlet.py 230152 2011-01-11 00:31:56Z dunlapm $
 
 from ZODB.POSException import ConflictError
 
@@ -8,18 +8,21 @@ from OFS import Moniker
 
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone.utils import safe_unicode
-
 from Products.Collage.interfaces import IDynamicViewManager
 from Products.Collage.utilities import getCollageSiteOptions
+from Products.Collage.utilities import CollageMessageFactory as _
 
+from zope.component import getMultiAdapter, queryUtility
+
+from plone.i18n.normalizer.interfaces import IIDNormalizer
 
 class SimpleContentMenuViewlet(object):
+    def isAlias(self):
+        return getattr(self.__parent__, '__alias__', None) is not None
+
     def portal_url(self):
         return getToolByName(self.context, 'portal_url')()
 
-    def test(self):
-        return lambda a, b, c: a and b or c
-
     def getImmediateObject(self):
         """Returns the immediate object added to the Collage
         instead of a possible alias target object."""
@@ -29,7 +32,6 @@ class SimpleContentMenuViewlet(object):
 
         return self.context.aq_inner
 
-
 class LayoutViewlet(SimpleContentMenuViewlet):
     def getLayouts(self):
         context = self.context
@@ -52,10 +54,16 @@ class LayoutViewlet(SimpleContentMenuViewlet):
         # filter out fallback view
         layouts = filter(lambda (name, title): name != u'fallback', layouts)
 
-        return [{'id': name,
-                 'name': title,
-                 'active': name == active} for (name, title) in layouts]
+        # make sure the active layout (which may not be available) is
+        # included
+        if active not in [name for name, title in layouts]:
+            layouts.append(
+                (active, _(u"Missing: $name", mapping={'name': active})))
 
+        return [{
+            'id': name,
+            'name': title,
+            'active': name == active} for (name, title) in layouts]
 
 class SkinViewlet(SimpleContentMenuViewlet):
     def getSkins(self):
@@ -89,16 +97,29 @@ class InsertNewItemViewlet(object):
         collage_options = getCollageSiteOptions()
         plone_view = self.context.restrictedTraverse('@@plone')
         container = plone_view.getCurrentFolder()
-
-        allowed_types = [t for t in container.getAllowedTypes()
-                         if collage_options.enabledType(t.getId())]
-
-        portal_url = getToolByName(self.context, 'portal_url')()
-
-        return [{'title': t.Title(),
-                 'description': t.Description(),
-                 'icon': '%s/%s' % (portal_url, t.getIcon()),
-                 'id': t.getId()} for t in allowed_types]
+        idnormalizer = queryUtility(IIDNormalizer)
+        
+        #BBB Plone 4
+        try:
+            factories_view = getMultiAdapter((self.context, self.request),
+                                             name='folder_factories')
+            allowed_types = [i for i in factories_view.addable_types() if
+                             collage_options.enabledType(i['id'])]
+            
+            return allowed_types
+        except:
+            portal_url = getToolByName(self.context, 'portal_url')()
+            results = []
+            for t in container.getAllowedTypes():
+                if collage_options.enabledType(t.getId()):
+                    cssId = idnormalizer.normalize(t.getId())
+                    cssClass = 'contenttype-%s' % cssId
+                    results.append({'title': t.Title(),
+                                    'description': t.Description(),
+                                    'icon': '%s/%s' % (portal_url, t.getIcon()),
+                                    'extra': {'id' : cssId, 'separator' : None,
+                                              'class' : cssClass},
+                                    'id': t.getId()})
 
 class SplitColumnViewlet(object):
     pass
@@ -111,16 +132,18 @@ class IconViewlet(SimpleContentMenuViewlet):
         return obj_typeinfo.getIcon()
 
 class ActionsViewlet(SimpleContentMenuViewlet):
-    def isAlias(self):
-
-        return getattr(self.__parent__, '__alias__', None) and True
 
     def getViewActions(self):
-
-        atool = getToolByName(self.context, 'portal_actions')
-        actions = atool.listFilteredActionsFor(self.context.aq_inner)
-        plone_view = self.context.restrictedTraverse('@@plone')
-        return plone_view.prepareObjectTabs()
+        plone_view = getMultiAdapter((self.context,self.request), name="plone")
+        try:
+            prepareObjectTabs = plone_view.prepareObjectTabs
+        except AttributeError:
+            provider = getMultiAdapter(
+                (self.context, self.request, plone_view),
+                name="plone.contentviews")
+            viewlet = provider.__getitem__("plone.contentviews")
+            prepareObjectTabs = viewlet.prepareObjectTabs
+        return prepareObjectTabs()
 
 class CopyViewlet(SimpleContentMenuViewlet):
     pass
@@ -144,7 +167,7 @@ class PasteViewlet(SimpleContentMenuViewlet):
         app = self.context.getPhysicalRoot()
         try:
             ob = m.bind(app)
-        except (ConflictError, AttributeError, KeyError):
+        except (ConflictError, AttributeError, KeyError, TypeError):
             return None
 
         return ob
@@ -166,4 +189,5 @@ class PasteViewlet(SimpleContentMenuViewlet):
         return u''
 
     def clipboard_item_title(self):
-        return u'Paste "%s"' % safe_unicode(self._get_clipboard_item().title_or_id())
+        title = self._get_clipboard_item().title_or_id()
+        return u'Paste "%s"' % safe_unicode(title)
