@@ -4,9 +4,9 @@ from zope.interface import (
 
 from zope.component import (
     getSiteManager, getMultiAdapter, getUtilitiesFor, queryUtility)
+from zope.component import ComponentLookupError
 
 from zope.annotation.interfaces import IAnnotations
-from zope.app.component.hooks import getSite
 from persistent.dict import PersistentDict
 
 from Products.Five import BrowserView
@@ -54,26 +54,26 @@ class DynamicViewManager(object):
     def getLayouts(self):
         context = self.context
         request = context.REQUEST
-        
+
         if ICollageAlias.providedBy(self.context):
             target = self.context.get_target()
             if target is not None:
                 context = target
 
-        ifaces = mark_request(request)
+        ifaces = mark_request(context, request)
 
         sm = getSiteManager()
         layouts = sm.adapters.lookupAll(
             required=(providedBy(context), providedBy(request)),
             provided=Interface
             )
-        
+
         directlyProvides(request, *ifaces)
-        layouts = [(name, getattr(layout, 'title', name)) 
+        layouts = [(name, getattr(layout, 'title', name))
                    for (name, layout) in layouts
-                   if type(layout) is type(BrowserView) 
-                      and issubclass(layout, BrowserView)
-                      and not getattr(layout, 'hide', False)]
+                   if isinstance(layout, type) and
+                      issubclass(layout, BrowserView) and
+                      not getattr(layout, 'hide', False)]
         layouts.sort(lambda a, b: cmp(a[1], b[1]))
         return layouts
 
@@ -95,7 +95,7 @@ class DynamicViewManager(object):
         if layout and request:
             request.debug = False
 
-            ifaces = mark_request(request)
+            ifaces = mark_request(self.context, request)
 
             target = self.context
             if ICollageAlias.providedBy(target):
@@ -103,9 +103,11 @@ class DynamicViewManager(object):
                 if not target:
                     target = self.context
 
-            view = getMultiAdapter((target, request), name=layout)
-
-            skinInterfaces = getattr(view, 'skinInterfaces', ())
+            try:
+                view = getMultiAdapter((target, request), name=layout)
+                skinInterfaces = getattr(view, 'skinInterfaces', ())
+            except ComponentLookupError:
+                skinInterfaces = ()
 
             for si in skinInterfaces:
                 for name, utility in getUtilitiesFor(si):
@@ -118,24 +120,23 @@ class DynamicViewManager(object):
         return skins
 
 
-def mark_request(request):
+def mark_request(context, request):
     """ Marks the request with general and theme-specific Collage browser layers.
-    
+
         Returns the initial set of request marker interfaces so that you can restore
         them using directlyProvides(request, ifaces).
     """
     initial_ifaces = directlyProvidedBy(request)
-    
+
     # general Collage layer
     directlyProvides(request, ICollageBrowserLayer)
-    
+
     # theme-specific Collage layer
-    site = getSite()
-    portal_skins = getToolByName(site, 'portal_skins', None)
+    portal_skins = getToolByName(context, 'portal_skins', None)
     if portal_skins is not None:
-        skin_name = site.getCurrentSkinName()
+        skin_name = portal_skins.getCurrentSkinName()
         layer_iface = queryUtility(ICollageBrowserLayerType, name=skin_name)
         if layer_iface is not None:
             directlyProvides(request, layer_iface, ICollageBrowserLayer)
-    
+
     return initial_ifaces
